// joi/10/juno/GUIShellConsole.java
//
//
// Copyright 2003 Bill Campbell and Ethan Bolker

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

/**
 *  The GUI to the Juno system Shell.
 */

public class GUIShellConsole extends JFrame 
    implements OutputInterface 
{
    private static final int FIELDWIDTH  = 50;
    private static final int FIELDHEIGHT = 10;

    // the components on the window

    private JLabel promptLabel = new JLabel();
    private JTextField commandLine = new JTextField( FIELDWIDTH );   
    private JButton doIt       = new JButton( "Do It" );    
    private JButton logout     = new JButton( "Logout" );  
    private JTextArea stdout   = 
                         new JTextArea( FIELDHEIGHT, FIELDWIDTH );
    private JTextArea stderr   = 
                         new JTextArea( FIELDHEIGHT/2, FIELDWIDTH );

    private Shell sh;    // for interpreting shell commands
    private WindowCloser closeMe; // for logging out.

    private boolean echoInput;

    /**
     * Construct a GUI console for a shell.
     *
     * @param title the title to  display in the frame.
     * @param sh the shell to interpret commands.
     * @param echoInput is input to be echoed?
     */

    public GUIShellConsole( String title, 
			    Shell sh,
			    boolean echoInput )
    { 
	this.sh = sh;
	this.echoInput   = echoInput;

	setTitle( title );
	setPrompt( sh.getPrompt() );

	// set up console's look and feel

	JPanel outerPanel = new JPanel();
	outerPanel.setLayout( new BorderLayout() );
		
	Box box = Box.createVerticalBox();

	JPanel commandPanel = new JPanel();
	commandPanel.setLayout( new BorderLayout() );
	commandPanel.add( promptLabel, BorderLayout.NORTH );
	commandPanel.add( commandLine, BorderLayout.CENTER );
	box.add( commandPanel );
	box.add( Box.createVerticalStrut( 10 ) );

	Box buttons = Box.createHorizontalBox();
	buttons.add( Box.createGlue() );
	buttons.add( doIt );
	buttons.add( Box.createGlue() );
	buttons.add( logout );
	buttons.add( Box.createGlue() );
	box.add( buttons );
	box.add( Box.createVerticalStrut( 10 ) );

	JPanel stdoutPanel = new JPanel();
        stdoutPanel.setLayout( new BorderLayout() );
        stdoutPanel.add( new JLabel( "Standard output:"), 
			 BorderLayout.NORTH ); 
			 
        stdoutPanel.add( new JScrollPane( stdout ),
			 BorderLayout.CENTER ); 
			 
        box.add( stdoutPanel );
        box.add( Box.createVerticalStrut( 10 ) );
	stdout.setEditable( false );

	JPanel stderrPanel = new JPanel();
        stderrPanel.setLayout( new BorderLayout() );
        stderrPanel.add( new JLabel( "Error output:" ),
			 BorderLayout.NORTH ); 
        stderrPanel.add( new JScrollPane( stderr ),
			 BorderLayout.CENTER ); 
        box.add( stderrPanel );
        box.add( Box.createVerticalStrut( 10 ) );
	stderr.setEditable( false );

	outerPanel.add( box, BorderLayout.CENTER );
	this.getContentPane().add( outerPanel, BorderLayout.CENTER );

	// Install menus and tool bar.

	JMenuBar menuBar  = new JMenuBar();
	JMenu commandMenu = new JMenu( "Command" );
	JMenu helpMenu    = new JMenu( "Help" );
	
	JToolBar toolBar  = new JToolBar();

	// Create menu items and tool buttons for each shell command

	ShellCommandTable table = sh.getSystem().getCommandTable();
	String [] commandNames = table.getCommandNames();
	for ( int i = 0; i < commandNames.length; i++ ) {

	    String commandName = commandNames[i];
	    ShellCommand command = 
		table.lookup( commandName );

	    CommandMenuAction commandAction = 
		new CommandMenuAction( commandName,
				       command.getArgString() );

	    HelpMenuAction helpAction = 
		new HelpMenuAction( commandName,
				    command.getArgString(),
				    command.getHelpString() );

	    JMenuItem item1 = commandMenu.add( commandAction );
	    JMenuItem item2 = helpMenu.add( helpAction );
	    JButton button  = toolBar.add( commandAction );
	    button.setToolTipText( command.getHelpString() );
	}

	this.setJMenuBar( menuBar );
	this.getContentPane().add( toolBar,
				   BorderLayout.NORTH ); 
	menuBar.add( commandMenu );
	menuBar.add( helpMenu );

	pack();
	show();

	// add Listener to the Do It button
	
	doIt.addActionListener( new Interpreter() );

	// add listener to the Logout button and window closer

	closeMe = new WindowCloser( this );
	logout.addActionListener( closeMe );
	this.addWindowListener( closeMe );
    }

    // Set the GUI prompt

    private void setPrompt(String prompt) 
    {
	this.promptLabel.setText(prompt);
    }

    // Implementing the OutputInterface. 
    // Everything goes to the single message area.

    public void println( String str ) 
    {
	stdout.append(str + "\n");
    }
    
    public void errPrintln( String str ) 
    {
	stderr.append(str + "\n");
    }
    
    public boolean isGUI() 
    {
	return true;
    }

    public boolean isRemote() 
    {
	return false;
    }

    public boolean isEchoInput() 
    {
	return echoInput;
    }

    // An inner class for the semantics when the user submits
    // a ShellCommand for execution.

    private class Interpreter 
        implements ActionListener 
    {
	public void actionPerformed( ActionEvent e ) 
	{
	    String str = commandLine.getText();
	    stdout.append( sh.getPrompt() + str + '\n');
	    if (sh.interpret( str )) {
		setPrompt( sh.getPrompt() );
	    }
	    else {
		closeMe.actionPerformed(null);
	    }
	}
    }

    private class CommandMenuAction extends AbstractAction 
    {
	private String argString;
	private String helpString;

	public CommandMenuAction( String text, String argString )
	{
	    super( text );
	    this. argString = argString;
	}

	public void actionPerformed( ActionEvent e )
	{
	    commandLine.setText( getValue( Action.NAME ) + 
				 " " + argString );
	}
    }

    private class HelpMenuAction extends AbstractAction 
    {
	private String argString;
	private String helpString;

	public HelpMenuAction( String text, String argString, 
			       String helpString )
	{
	    super( text );
	    this. argString = argString;
	    this.helpString = helpString;
	}

	public void actionPerformed( ActionEvent e )
	{
	    stdout.append( getValue( Action.NAME ) + ": " +
			   helpString );
	}
    }
    
    //  A WindowCloser instance handles close events generated 
    //  by the underlying window system with its windowClosing
    //  method, and close events from buttons or other user
    //  components with its actionPerformed method.
    //
    //  The action is to logout and dispose of this window.

    private static class WindowCloser extends WindowAdapter
        implements ActionListener  
    {
        Frame myFrame;

        public WindowCloser( Frame frame ) {
            myFrame = frame;
        }

        public void windowClosing (WindowEvent e) 
	{
            this.actionPerformed( null ); 
        }

        public void actionPerformed(ActionEvent e) 
	{
	    myFrame.dispose();
        }
    }
} 
