// joi/5/jfiles/JFile.java 
//
//
// Copyright 2003 Bill Campbell and Ethan Bolker

import java.util.Date;
import java.io.File;

/**
 * A JFile object models a file in a hierarchical file system.
 * <p>
 * Extend this abstract class to create particular kinds of JFiles, 
 * e.g.:<br>
 * Directory  - 
 *    a JFile that maintains a list of the files it contains.<br>
 * TextFile  - 
 *    a JFile containing text you might want to read.<br>
 *
 * @see Directory
 * @see TextFile
 *
 * @version 5
 */

public abstract class JFile 
{
    /**
     * The separator used in pathnames.
     */

    public static final String separator = File.separator;

    private String    name;       // a JFile knows its name
    private String    owner;      // the owner of this file
    private Date      createDate; // when this file was created
    private Date      modDate;    // when this file was last modified
    private Directory parent;     // the Directory containing this file

    /**
     * Construct a new JFile, set owner, parent, creation and
     * modification dates. Add this to parent (unless this is the
     * root Directory).
     *
     * @param name     the name for this file (in its parent directory).
     * @param creator  the owner of this new file.
     * @param parent   the Directory in which this file lives.
     */

    protected JFile( String name, String creator, Directory parent ) 
    {
        this.name   = name;
        this.owner  = creator;
        this.parent = parent;
        if (parent != null) {
            parent.addJFile( name, this );
        }
        createDate  = modDate = new Date(); // set dates to now
    }

    /** 
     * The name of the file.
     *
     * @return the file's name.
     */
    
    public String getName() 
    {
        return name;
    }

    /**
     * The full path to this file.
     *
     * @return the path name.
     */

    public String getPathName()
    {
        if (this.isRoot()) {
            return separator;
        }
        if (parent.isRoot()) {
            return separator + getName();
        }
        return parent.getPathName() + separator + getName();
    }

    /**
     * The size of the JFile 
     * (as defined by the child class)..
     *
     * @return the size.
     */

    public abstract int getSize();

    /**
     * Suffix used for printing file names
     * (as defined by the child class).
     *
     * @return the file's suffix.
     */

    public abstract String getSuffix();
    
    /** 
     * Set the owner for this file.
     *
     * @param owner the new owner.
     */

    public void setOwner( String owner )
    {
        this.owner = owner;
    }
    
    /**
     * The file's owner.
     *
     * @return the owner of the file.
     */

    public String getOwner() 
    {
        return owner;
    }

    /**
     * The date and time of the file's creation.
     *
     * @return the file's creation date and time.
     */

    public String getCreateDate() 
    {
        return createDate.toString();
    }
    
    /**
     * Set the modification date to "now".
     */

    protected void setModDate() 
    {
        modDate = new Date();
    }

    /**
     * The date and time of the file's last modification.
     *
     * @return the date and time of the file's last modification.
     */

    public String getModDate() 
    {
        return modDate.toString();
    }

    /**
     * The Directory containing this file.
     *
     * @return the parent directory.
     */

    public Directory getParent()
    {
        return parent;
    }

    /**
     * A JFile whose parent is null is defined to be the root
     * (of a tree). 
     *
     * @return true when this JFile is the root.
     */

    public boolean isRoot()
    {
        return (parent == null);
    }

    /**
     * How a JFile represents itself as a String. 
     * That is,
     * <pre>
     *    owner   size    modDate    name+suffix
     * </pre>
     *
     * @return the String representation.
     */

    public String toString() 
    {
        return getOwner() + "\t" + 
            getSize()  + "\t" + 
            getModDate()  + "\t" + 
            getName() + getSuffix();
    }

    // Unit test: main() and static support

    private static Terminal terminal = new Terminal();

    /**
     * A unit test of JFile and its subclasses.
     */

    public static void main( String[] args )
    {
        out("Some hardwired, self documenting JFile system tests");
        out("create and then explore JFile hierarchy");
        out("   root                (owner sysadmin)");
        out("       billhome        (owner bill)");
        out("       ebhome          (owner eb)");
        out("           cs110       (owner eb)");
        out("               diary   (owner eb)");
        out("           insult      (owner bill)");
        Directory root   = new Directory( "",     "sysadmin", null );
        Directory home1  = new Directory( "ebhome",   "eb",   root );
        Directory home2  = new Directory( "billhome", "bill", root );

        TextFile  insult = new TextFile( "insult", "bill", home1,
                                         "Your mother wore sneakers,");
        insult.append( "\nin the shower." );

        Directory cs110  = new Directory( "cs110", "eb", home1);
        cs110.addJFile( "diary",
                        new TextFile( "diary", "eb", cs110,
                                      "started work on Chapter 3"));

        out("\nlist contents of the root directory:" );
        list( root );

        out("\nlist contents of ebhome:" );
        list( home1 );

        out("\nretrieve billhome, list its contents (empty)):");
        list( (Directory) root.retrieveJFile("billhome") );

        out("\nretrieve insult, contents two line insult:");
        type( (TextFile)home1.retrieveJFile("insult"));

        out("\nretrieve file \"foo\" from ebhome, try to display it:");
        type( (TextFile)home1.retrieveJFile("foo") );

        out("\nlist contents of cs110 (one file):" );
        list( (Directory) home1.retrieveJFile("cs110") );

        out("path to root:\t "   + root.getPathName() );
        out("path to ebhome:\t " + home1.getPathName() );
        out("path to cs110:\t "  + cs110.getPathName() );
     }

     // display a listing of the contents of a Directory

     private static void list( Directory dir )
     {
         terminal.println( dir.getName() );
         terminal.println( dir.getSize() +
                           (dir.getSize() == 1
                           ? " file:" : " files:") );

         String[] fileNames = dir.getFileNames();
         for ( int i = 0; i < fileNames.length; i++ ) {
             String fileName = fileNames[i];
             JFile  jfile    = dir.retrieveJFile( fileName );
             terminal.println( jfile.toString() ); 
         }
     }

     // display the contents of a TextFile 

     private static void type( TextFile file )
     {
        String whatToPrint;
        if (file == null) {
            whatToPrint = "no such file";
        }
        else {
            whatToPrint = file.getContents();
        }
        terminal.println( whatToPrint );
    }

    // abbreviation for "terminal.println"

    private static void out( String s )
    {
        terminal.println( s );
    }
}
